<?php
require_once '../db/config.php';

// Custom exception for payment-specific errors
class PaymentException extends Exception {}

// Configure error handling
error_reporting(E_ALL);
ini_set('display_errors', 0);

// Set response headers
header("Content-Type: application/json; charset=utf-8");
header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");
header("Cache-Control: post-check=0, pre-check=0", false);
header("Pragma: no-cache");

function createDatabaseConnection() {
    global $servername, $username, $password, $dbname;

    $conn = new mysqli($servername, $username, $password, $dbname);
    if ($conn->connect_error) {
        throw new Exception('Database connection failed: ' . $conn->connect_error);
    }
   


    return $conn;
    

}

function sendResponse($data, $statusCode = 200) {
    http_response_code($statusCode);
    echo json_encode($data, JSON_PRETTY_PRINT);
    exit;
}

function sendErrorResponse($message, $code = 400, $details = null) {
    $response = [
        'success' => false,
        'message' => $message,
        'debug' => [
            'request_method' => $_SERVER['REQUEST_METHOD'],
            'post_data' => $_POST,
            'timestamp' => date('Y-m-d H:i:s'),
            'details' => $details
        ]
    ];

    sendResponse($response, $code);
}

function validateInput($vendorId, $userId, $amount, $conceptoId) {
    if (!is_numeric($vendorId) || !is_numeric($userId) || !is_numeric($amount) || !is_numeric($conceptoId)) {
        throw new PaymentException('Invalid input format');
    }

    if ($vendorId <= 0 || $userId <= 0 || $amount <= 0 || $conceptoId <= 0) {
        throw new PaymentException('All input values must be positive integers');
    }
}

function processPayment($conn, $vendorId, $userId, $amount, $conceptoId, $notas) {
    try {
        // Start transaction
        $conn->begin_transaction();

        // Insert payment record
        $insertQuery = "
            INSERT INTO unique_payments (vendor_id, amount, concepto_id, notas, payment_date, payment_time, created_at, user_id)
            VALUES (?, ?, ?, ?, CURDATE(), CURTIME(), NOW(), ?)
        ";

        $stmt = $conn->prepare($insertQuery);
        if (!$stmt) {
            throw new Exception('Failed to prepare insert query: ' . $conn->error);
        }

        $stmt->bind_param("didsi", $vendorId, $amount, $conceptoId, $notas, $userId);

        if (!$stmt->execute()) {
            throw new Exception('Failed to execute insert query: ' . $stmt->error);
        }

        $paymentId = $stmt->insert_id;

        if (!$paymentId) {
            throw new Exception('Failed to retrieve payment ID');
        }

        // Get user details
        $userQuery = "SELECT name FROM users WHERE id = ?";
        $stmt = $conn->prepare($userQuery);
        if (!$stmt) {
            throw new Exception('Failed to prepare user query: ' . $conn->error);
        }

        $stmt->bind_param("i", $userId);
        $stmt->execute();
        $result = $stmt->get_result();
        $userName = $result->fetch_assoc()['name'] ?? 'Unknown User';

        // Commit transaction
        $conn->commit();

        // Prepare success response
        $response = [
            'success' => true,
            'message' => 'Payment processed successfully',
            'data' => [
                'unique_payment_id' => $paymentId,  // Changed from 'payment_id' to 'unique_payment_id'
                'amount' => $amount,
                'user_name' => $userName,
                'vendor_id' => $vendorId,
                'notas' => $notas,
                'payment_date' => date('Y-m-d'),
                'payment_time' => date('H:i:s')
            ]
        ];

        return $response;

    } catch (Exception $e) {
        // Rollback transaction on error
        $conn->rollback();
        throw $e;
    }
}

// Main execution
try {
    // Log the request method and other details
    error_log("Request Method: " . $_SERVER['REQUEST_METHOD']);
    error_log("POST Data: " . print_r($_POST, true));

    if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
        throw new PaymentException('Invalid request method', 405);
    }

    if (!isset($_POST['vendor_id']) || !isset($_POST['user_id']) || !isset($_POST['amount']) || !isset($_POST['concepto_id']) || !isset($_POST['notas'])) {
        throw new PaymentException('Missing required parameters: vendor_id, user_id, amount, concepto_id, notas');
    }

    $vendorId = filter_input(INPUT_POST, 'vendor_id', FILTER_VALIDATE_INT);
    $userId = filter_input(INPUT_POST, 'user_id', FILTER_VALIDATE_INT);
    $amount = filter_input(INPUT_POST, 'amount', FILTER_VALIDATE_FLOAT);
    $conceptoId = filter_input(INPUT_POST, 'concepto_id', FILTER_VALIDATE_INT);
    $notas = filter_input(INPUT_POST, 'notas', FILTER_SANITIZE_STRING);

    validateInput($vendorId, $userId, $amount, $conceptoId);

    $conn = createDatabaseConnection();

    // Set timeout and transaction isolation level
    $conn->query("SET SESSION wait_timeout=30");
    $conn->query("SET SESSION TRANSACTION ISOLATION LEVEL SERIALIZABLE");

    $response = processPayment($conn, $vendorId, $userId, $amount, $conceptoId, $notas);
    $conn->close();

    sendResponse($response);

} catch (PaymentException $e) {
    sendErrorResponse($e->getMessage(), $e->getCode(), $e->getMessage());
} catch (Exception $e) {
    error_log("Payment Processing Error: " . $e->getMessage());
    sendErrorResponse('An internal error occurred', 500, $e->getMessage());
} finally {
    if (isset($conn) && $conn instanceof mysqli) {
        $conn->close();
    }
}
?>